<?php

namespace Drupal\stripe_webform\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformElementBase;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Provides a 'stripe' element.
 *
 * @WebformElement(
 *   id = "stripe",
 *   label = @Translation("Stripe"),
 *   category = @Translation("Stripe"),
 *   composite = TRUE,
 *   description = @Translation("Provides a placeholder for a stripe elements integration."),
 * )
 */
class StripeWebformElement extends WebformElementBase {

  protected $stripeWebformElementDefaultProperties = [
    'stripe_currency' => 'usd',
    'stripe_paymentrequest' => TRUE,
    'stripe_label' => '',
    'stripe_country' => 'US',
    'stripe_name' => '',
    'stripe_email' => '',
    'stripe_billing_address1' => '',
    'stripe_billing_address2' => '',
    'stripe_billing_city' => '',
    'stripe_billing_state' => '',
    'stripe_billing_country' => '',
    'webform_stripe_subscriptions' => FALSE,
    'webform_stripe_amount' => '',
  ];

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    parent::prepare($element, $webform_submission);

    foreach ($this->stripeWebformElementDefaultProperties as $key => $value) {
      $element["#$key"] = $this->getElementProperty($element, $key);
    }

    if (empty($element['#webform_stripe_amount']) || !is_numeric($element['#webform_stripe_amount'])) {
      $element['#stripe_amount'] = (int) $this->getDefaultProperty('webform_stripe_amount');
    }
    else {
      $element['#stripe_amount'] = (int) $element['#webform_stripe_amount'];
    }

    if (empty($element['#stripe_label'])) {
      $element["#stripe_label"] = $webform_submission->getWebform()->label();
    }

    if (!empty($element['#webform_stripe_subscriptions'])) {
      $element["#stripe_paymentintent"]['setup_future_usage'] = 'off_session';
    }

  }

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties() {
    $info = $this->getInfo();
    // Matching defaults from the element.
    $this->stripeWebformElementDefaultProperties['stripe_currency'] = $info['#stripe_currency'];
    $this->stripeWebformElementDefaultProperties['stripe_country'] = $info['#stripe_country'];
    // Make our own amount a string so we can safely store tokens
    $this->stripeWebformElementDefaultProperties['webform_stripe_amount'] = (string) $info['#stripe_amount'];
    return $this->stripeWebformElementDefaultProperties + parent::defineDefaultProperties();
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['stripe'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Stripe settings'),
      '#description' => $this->t('You can use tokens (including webform:submission tokens) to dynamically set this values before processing payment.'),
    ];

    $form['stripe']['stripe_country'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Country'),
      '#description' => $this->t('The two-letter country code of your Stripe account (e.g., US).')
    ];

    $form['stripe']['stripe_currency'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Currency'),
      '#description' => $this->t('Three character currency code (e.g., usd).')
    ];

    $form['stripe']['stripe_paymentrequest'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Adds a Payment Request button'),
      '#description' => $this->t('If supported by the browser, show a Pay button alongside the credit card field.'),
      '#return_type' => TRUE,
    ];

    $form['stripe']['webform_stripe_subscriptions'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable subscriptions'),
      '#description' => $this->t('If you are going to use the submit handler to create monthly subscriptions, you need to enable it so that the captured payment information can be reused.'),
      '#return_type' => TRUE,
    ];

    $form['stripe']['stripe_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#description' => $this->t('It will be used as a placeholder for the total if needed by stripe (i.e. Payment Request Buttons). If blank, the webform title will be used.'),
    ];

    $form['stripe']['webform_stripe_amount'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Amount'),
      '#description' => $this->t('Amount in dollars (not cents) the card will be charged. You can use tokens.'),
    ];

    $form['stripe']['stripe_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
    ];

    $form['stripe']['stripe_email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('E-mail'),
    ];

    $form['stripe']['billing_details'] = [
      '#type' => 'details',
      '#title' => $this->t('Billing details'),
      '#open' => TRUE,
    ];

    $form['stripe']['billing_details']['stripe_billing_address1'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Address Line 1'),
    ];

    $form['stripe']['billing_details']['stripe_billing_address2'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Address Line 2'),
    ];

    $form['stripe']['billing_details']['stripe_billing_city'] = [
      '#type' => 'textfield',
      '#title' => $this->t('City'),
    ];

    $form['stripe']['billing_details']['stripe_billing_state'] = [
      '#type' => 'textfield',
      '#title' => $this->t('State'),
    ];

    $form['stripe']['billing_details']['stripe_billing_country'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Country'),
    ];

    return $form;
  }

}
