<?php

namespace Drupal\stripe_webform\Plugin\WebformHandler;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\webform\Utility\WebformYaml;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformTokenManagerInterface;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission stripe handler.
 *
 * @WebformHandler(
 *   id = "stripe",
 *   label = @Translation("Stripe"),
 *   category = @Translation("Stripe"),
 *   description = @Translation("Creates subscription and other stripe objects."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 * )
 */
class StripeWebformHandler extends WebformHandlerBase {

    /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    $instance->tokenManager = $container->get('webform.token_manager');
    return $instance;
  }
  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'amount' => '',
      'stripe_element' => '',
      'price_id' => '',
      'quantity' => '',
      'currency' => 'usd',
      'metadata' => '',
      'stripe_customer_create' => '',
      'stripe_charge_create' => '',
      'stripe_subscription_create' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $webform = $this->getWebform();

    $elements = $webform->getElementsInitializedFlattenedAndHasValue('view');
    $options = [];
    foreach ($elements as $key => $element) {
      if ($element['#type'] == 'stripe') {
        $options[$key] = $element['#admin_title'] ?: $element['#title'] ?: $key;
      }
    }

    $form['stripe'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Stripe settings'),
    ];

    $form['stripe']['stripe_element'] = [
      '#type' => 'select',
      '#title' => $this->t('Stripe element'),
      '#required' => TRUE,
      '#options' => ['' => $this->t('-Select-')] + $options,
      '#parents' => ['settings', 'stripe_element'],
      '#default_value' => $this->configuration['stripe_element'] ?: (count($options) == 1 ? $key : ''),
    ];

    $form['stripe']['amount'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Amount'),
      '#default_value' => $this->configuration['amount'],
      '#parents' => ['settings', 'amount'],
      '#description' => $this->t('Amount to charge the credit card. You may use tokens.'),
      '#required' => TRUE,
    ];

    $form['stripe']['subscription'] = [
      '#type' => 'details',
      '#title' => t('Subscriptions'),
      '#description' => $this->t('Optional fields to subscribe the customer to a plan instead of a directly charging it.'),
    ];
    $form['stripe']['subscription']['price_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Price ID'),
      '#default_value' => $this->configuration['price_id'],
      '#parents' => ['settings', 'price_id'],
      '#description' => $this->t('Stripe subscriptions price_ id. You may use tokens.'),
    ];
    $form['stripe']['subscription']['quantity'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Quantity'),
      '#default_value' => $this->configuration['quantity'],
      '#parents' => ['settings', 'quantity'],
      '#description' => $this->t('Quantity of the plan to subscribe. You may use tokens.'),
    ];

    $form['stripe']['metadata_details'] = [
      '#type' => 'details',
      '#title' => $this->t('Meta data'),
      '#description' => $this->t('Additional <a href=":url" target="_blank">metadata</a> in YAML format, each line a <em>key: value</em> element. You may use tokens.', [':url' => 'https://stripe.com/docs/api#metadata']),
    ];

    $form['stripe']['metadata_details']['metadata'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Meta data'),
      '#parents' => ['settings', 'metadata'],
      '#default_value' => $this->configuration['metadata'],
    ];

    $form['stripe']['advanced'] = [
      '#type' => 'details',
      '#title' => t('Advanced settings'),
      '#open' => FALSE,
    ];
    $form['stripe']['advanced']['stripe_customer_create'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Customer create object'),
      '#parents' => ['settings', 'stripe_customer_create'],
      '#default_value' => $this->configuration['stripe_customer_create'],
      '#description' => $this->t('Additional fields of the stripe API call to <a href=":url" target="_blank">create a customer</a>. You cannot override the keys set by the fields above.', [':url' => 'https://stripe.com/docs/api#create_customer']),
    ];
    $form['stripe']['advanced']['stripe_charge_create'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Charge create object'),
      '#parents' => ['settings', 'stripe_charge_create'],
      '#default_value' => $this->configuration['stripe_charge_create'],
      '#description' => $this->t('Additional fields of the stripe API call to <a href=":url" target="_blank">create a charge</a>. You cannot override the keys set by the fields above.', [':url' => 'https://stripe.com/docs/api#create_charge']),
    ];
    $form['stripe']['advanced']['stripe_subscription_create'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Subscription create object'),
      '#parents' => ['settings', 'stripe_subscription_create'],
      '#default_value' => $this->configuration['stripe_subscription_create'],
      '#description' => $this->t('Additional fields of the stripe API call to <a href=":url" target="_blank">create a subscription</a>. You cannot override the keys set by the fields above.', [':url' => 'https://stripe.com/docs/api#create_subscription']),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValues();
    foreach ($this->configuration as $name => $value) {
      if (isset($values[$name])) {
        $this->configuration[$name] = $values[$name];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    // If update do nothing
    if ($update) {
      return;
    }

    $uuid = $this->configFactory->get('system.site')->get('uuid');

    $config = $this->configFactory->get('stripe.settings');
    $apikeySecret = $config->get('apikey.' . $config->get('environment') . '.secret');

    // Replace tokens.
    $data = $this->tokenManager->replace($this->configuration, $webform_submission);

    $webform = $webform_submission->getWebform();
    $element = $webform->getElement($data['stripe_element']);
    $element_manager = \Drupal::service('plugin.manager.webform.element');
    $element_plugin = $element_manager->getElementInstance($element, $webform_submission);
    $element_plugin->prepare($element, $webform_submission);

    try {
      $stripe = new \Stripe\StripeClient($apikeySecret);

      $metadata = [
        'uuid' => $uuid,
        'webform' => $webform_submission->getWebform()->label(),
        'webform_id' => $webform_submission->getWebform()->id(),
        'webform_submission_id' => $webform_submission->id(),
      ];

      if (!empty($data['metadata'])) {
        $metadata += Yaml::decode($data['metadata']);
      }

      $payment_intent = $stripe->paymentIntents->retrieve($webform_submission->getElementData($data['stripe_element'])['payment_intent']);
      $payment_method = $stripe->paymentMethods->retrieve($payment_intent->payment_method);

       // Create a Customer:
      $stripe_customer_create = [
        'name' => $payment_method->billing_details->name,
        'email' => $payment_method->billing_details->email,
        'address' => $payment_method->billing_details->address->toArray(),
        'payment_method' => $payment_intent->payment_method,
        'invoice_settings' => [
          'default_payment_method' => $payment_intent->payment_method,
        ],
        'metadata' => $metadata,
      ];
      if (!empty($data['stripe_customer_create'])) {
        $stripe_customer_create += Yaml::decode($data['stripe_customer_create']);
      }
      $customer = $stripe->customers->create($stripe_customer_create);

      if (!empty($data['price_id'])) {
        $price = $stripe->prices->retrieve($data['price_id']);
        $interval = "+{$price->recurring->interval_count} {$price->recurring->interval}";
        $anchor = strtotime($interval);

        $stripe_subscription_create = [
          'customer' => $customer->id,
          'items' => [
            [
              'price' => $price->id,
              'quantity' => $data['quantity'] ?: 1,
            ],
          ],
          'metadata' => $metadata,
          'billing_cycle_anchor' => $anchor,
          'proration_behavior' => 'none',
        ];
        if (!empty($data['stripe_subscription_create'])) {
          $stripe_subscription_create += Yaml::decode($data['stripe_subscription_create']);
        }

        $stripe->subscriptions->create($stripe_subscription_create);
      }
    }
    catch (\Stripe\Exception\ExceptionInterface $e) {
      $this->messenger()->addError($this->t('Stripe error: %error', ['%error' => $e->getMessage()]), 'error');
    }
 }

}
